function [D, X, nopt, vn, rmsev] = itc_adl(Y, D0, s, nmin, iters, iter_check, nminus, nplus, itc_index, ncand)

% Input:
%   Y           - training signals matrix
%   D0          - initial dictionary
%   s           - sparsity level
%   nmin        - minimum number of atoms
%   iters       - total number of DL iterations
%   iter_check  - number of iterations at which size may be changed
%   nminus      - how much n is decreased (when decreased)
%   nplus       - how much n is increased (when increased)
%   itc_index   - number of ITC (3=BIC, 4=RNML1, 5=RNML2) - old versions
%               - 11,12-BIC, 13,14-RNML (first is extended ITC, second is approximation of it
%   ncand       - number of size candidates for which ITC is computed
% Output:
%   D           - learned dictionary
%   X           - representation matrix
%   nopt        - optimal size as indicated by ITC (vector of iters/iter_check elements)
%   vn          - size set by itc_adl (vector of iters/iter_check elements)
%   rmsev       - RMSE history

% BD 19.05.2018

if nargin < 10
  ncand = 1000;  % big number to make it sure that there are enough candidates
end

% parameters that might be changed
update = 'aksvd';       % set the dictionary update method
                        % test with 'nsgk' ???
replatom = 'random';    % unused atoms are replaced with random vectors
params = {};

[m,n0] = size(D0);
[~,N] = size(Y);
itcmat = zeros(iters/iter_check, n0-nmin+1);
D0run = D0;
ii = 1;
vn = [n0];    % vector of actual sizes
for it = 1 : iter_check : iters
  % run the algorithm for a few iterations
  [D, X, ~] = DL(Y, D0run, s, iter_check, str2func(update), params, 'replatom', replatom);
  
  % order dictionary based on representation power
  [~, atom_list] = sort(sum(X.*X,2), 'descend');
  
  % compute ITC for all dictionary sizes (with ordered atoms)
  n1 = max(nmin, vn(ii) - ncand);
  for n = n1 : vn(ii)
    Dn = D(:, atom_list(1:n));
    Xn = omp(Y, Dn, s);
    rmse = norm(Y-Dn*Xn, 'fro')/sqrt(m*N);
    if itc_index > 10
      itc2 = com_itc_BS_2( Y, Dn, Xn, rmse );
      itcmat(ii,n-n1+1) = itc2(itc_index-10);
    else
      [~, itc2] = com_itc_BS( Y, Dn, Xn, rmse );
      itcmat(ii,n-n1+1) = itc2(itc_index);
    end
  end
  
  % trim or enlarge dictionary
  [~,n] = min(itcmat(ii,1:vn(ii)-n1+1));   % compute min value of ITC
  n = n + n1 - 1;
  nopt(ii) = n;
  itcmat(ii,vn(ii)-n1+2:n0-n1+1) = itcmat(ii,vn(ii)-n1+1); % fill with last value of ITC
  
  if n < vn(ii) - nminus            % dictionary is too big
    n = max(nmin, vn(ii) - nminus);
    D0run = D(:, atom_list(1:n));    % continue with new dictionary
  elseif n < vn(ii)-1
    n = max(nmin, vn(ii) - 1);
    D0run = D(:, atom_list(1:n));    % continue with new dictionary
  elseif n == vn(ii)   % dictionay may be too small, increase size
    %n = vn(ii);
    n = vn(ii) + nplus;
    Dplus = randn(m,nplus);
    Dplus = normc(Dplus);
    D0run = [D Dplus];    % continue with new dictionary
  end
  vn(ii+1) = n;

  % final operations
  X = omp(Y, D0run, s);        % compute RMSE just to see it
  rmse = norm(Y-D0run*X, 'fro')/sqrt(m*N);
  rmsev(ii) = rmse;
  
  ii = ii+1;
end

% final refinement: a few DL iterations with the final size
if nopt(ii-1) < vn(ii-1)
  D0run = D(:, atom_list(1:nopt(ii-1)));    % cut dictionary to optimal number of atoms
end
[D, ~, ~] = DL(Y, D0run, s, iter_check, str2func(update), params, 'replatom', replatom);
X = omp(Y, D, s);
rmse = norm(Y-D*X, 'fro')/sqrt(m*N);

